open Attr;;

let rec make_break_ctx ctx list = 
  match list with 
      [] -> ctx
    | tag::l ->
	let restctx = make_break_ctx ctx l in
	  match tag with 
	      "B"  -> { restctx with b = (False, 1) }
	    | "EM" -> { restctx with em = (False, 1) }
	    | "I"  -> { restctx with i = (False, 1) }
	    | "PL" -> { restctx with pl = (False, 1) }
	    | "S"  -> { restctx with s = (False, 1) }
	    | "TT" -> { restctx with tt = (False, 1) }
	    | "U"  -> 
		let (u1s, _) = restctx.u1 in
		let (u2s, _) = restctx.u2 in
		let (u3s, _) = restctx.u3 in
		  if (u3s = False) then 
		    (if (u2s = False) then 
		      { restctx with u1 = (False, 1) }
		    else 
		      { restctx with u2 = (False, 1) }
		    )
		  else 
		      { restctx with u3 = (True, 1) }		      
	    | "r"  -> { restctx with clr = (False, 1) }
	    | "g"  -> { restctx with clg = (False, 1) }
	    | "b"  -> { restctx with clb = (False, 1) }
	    | "c"  -> { restctx with clc = (False, 1) }
	    | "y"  -> { restctx with cly = (False, 1) }
	    | "m"  -> { restctx with clm = (False, 1) }
	    | "k"  -> { restctx with clk = (False, 1) }
	    | "w"  -> { restctx with clw = (False, 1) }
	    | "0"  -> { restctx with z0 = (False, 1) }
	    | "1"  -> { restctx with z1 = (False, 1) }
	    | "2"  -> { restctx with z2 = (False, 1) }
	    | "3"  -> { restctx with z3 = (False, 1) }
	    | "4"  -> { restctx with z4 = (False, 1) }
	    | "5"  -> { restctx with z5 = (False, 1) }
	    | "6"  -> { restctx with z6 = (False, 1) }
	    | "7"  -> { restctx with z7 = (False, 1) }
	    | "8"  -> { restctx with z8 = (False, 1) }
	    | "9"  -> { restctx with z9 = (False, 1) }
	    | _ -> restctx
;;

let diff ctx octx =
  let diffboolstep oldon oldcarry attr oldattr tag =
    let (ft,fi) = attr in
    let (oft,ofi) = oldattr in
    let newcarry = 
      if (ft = True && oft = True) then ((tag,fi)::oldcarry)
      else oldcarry
    in
      if (ft = oft) then (oldon, newcarry)
      else match ft with
	  True  -> ((tag, fi)::oldon, newcarry)
	| False -> (oldon, newcarry)
	| _  -> (oldon, newcarry) (* raise RenderError *)
  in
  let (bon, bcarry)       = diffboolstep  []    []        ctx.b    octx.b   "B"  in
  let (emon, emcarry)     = diffboolstep  bon   bcarry    ctx.em   octx.em  "EM" in
  let (ion, icarry)       = diffboolstep  emon  emcarry   ctx.i    octx.i   "I"  in
  let (plon, plcarry)     = diffboolstep  ion   icarry    ctx.pl   octx.pl  "PL" in
  let (son, scarry)       = diffboolstep  plon  plcarry   ctx.s    octx.s   "S"  in
  let (tton, ttcarry)     = diffboolstep  son   scarry    ctx.tt   octx.tt  "TT" in
  let (u1on, u1carry)     = diffboolstep  tton  ttcarry   ctx.u1   octx.u1  "U"  in
  let (u2on, u2carry)     = diffboolstep  u1on  u1carry   ctx.u2   octx.u2  "U"  in
  let (u3on, u3carry)     = diffboolstep  u2on  u2carry   ctx.u3   octx.u3  "U"  in
  let (clron, clrcarry)   = diffboolstep  u3on  u3carry   ctx.clr  octx.clr "r"  in
  let (clgon, clgcarry)   = diffboolstep  clron clrcarry  ctx.clg  octx.clg "g" in
  let (clbon, clbcarry)   = diffboolstep  clgon clgcarry  ctx.clb  octx.clb "b" in
  let (clcon, clccarry)   = diffboolstep  clbon clbcarry  ctx.clc  octx.clc "c" in
  let (clyon, clycarry)   = diffboolstep  clcon clccarry  ctx.cly  octx.cly "y" in
  let (clmon, clmcarry)   = diffboolstep  clyon clycarry  ctx.clm  octx.clm "m" in
  let (clkon, clkcarry)   = diffboolstep  clmon clmcarry  ctx.clk  octx.clk "k" in
  let (clwon, clwcarry)   = diffboolstep  clkon clkcarry  ctx.clw  octx.clw "w" in
  let (sz0on, sz0carry)   = diffboolstep  clwon clwcarry  ctx.z0   octx.z0  "0" in
  let (sz1on, sz1carry)   = diffboolstep  clwon clwcarry  ctx.z1   octx.z1  "1" in
  let (sz2on, sz2carry)   = diffboolstep  clwon clwcarry  ctx.z2   octx.z2  "2" in
  let (sz3on, sz3carry)   = diffboolstep  clwon clwcarry  ctx.z3   octx.z3  "3" in
  let (sz4on, sz4carry)   = diffboolstep  clwon clwcarry  ctx.z4   octx.z4  "4" in
  let (sz5on, sz5carry)   = diffboolstep  clwon clwcarry  ctx.z5   octx.z5  "5" in
  let (sz6on, sz6carry)   = diffboolstep  clwon clwcarry  ctx.z6   octx.z6  "6" in
  let (sz7on, sz7carry)   = diffboolstep  clwon clwcarry  ctx.z7   octx.z7  "7" in
  let (sz8on, sz8carry)   = diffboolstep  clwon clwcarry  ctx.z8   octx.z8  "8" in
  let (sz9on, sz9carry)   = diffboolstep  clwon clwcarry  ctx.z9   octx.z9  "9" in
  let ordering a b =
    let (astr, ai) = a in
    let (bs, bi) = b in
    let orderInt = compare ai bi in
      if orderInt = 0 then
	compare astr bs
      else
	orderInt in
    (List.sort ordering sz9on, sz9carry)
    
let overlap_resolution doc =
  let rec overlap_resolution_step doc ctx = 
    match doc with 
      | ((s1, ctx1)::rest) -> 
	  let (on, carry) = diff ctx1 ctx in
	  let rec print_attrs l
	    = match l with
		[] -> ();
	      | h::t -> print_string (attv h); print_string ", "; print_int (attl h); print_attrs t
	  in
	  let breaks newtags = 
	    match newtags with 
		[] -> []
	      | (h::l) -> 
		  let r0 = attl h in
		  let rec check_new carrytags =
		    match carrytags with 
			[] -> []
		      | (h2::l2) ->
			  let r = attl h2 in
			    if (r0 > r) then 
			      ( 
			    (attv h2)::check_new l2
			      )
			    else check_new l2
		  in
		    (check_new carry) @ []
	  in
	  let bs = breaks on in
	  let newctx = make_break_ctx ctx bs in
	  let rest = (s1, ctx1)::overlap_resolution_step rest ctx1 in
	    if bs = [] then 
	      rest 
	    else
	      ("", newctx)::rest
      | e -> e
  in
    overlap_resolution_step doc default
;;









