open Attr;;
exception RenderError;;

let render table = 
  let sbuf = ref "" in
  let rec render2 table octx stack =
    let diff ctx octx =
      let diffboolstep oldon oldoff attr oldattr tag =
	let (ft,fi) = attr in
	let (oft,ofi) = oldattr in
	  if ft = oft then
	    (oldon, oldoff)
	  else
	    match ft with
		True  -> ((tag, fi)::oldon, oldoff)
	      | False -> (oldon, tag::oldoff)
	      | _  -> (oldon, oldoff) (* raise RenderError *)
      in
      let (bon, boff)       = diffboolstep  []    []      ctx.b    octx.b   "B"  in
      let (emon, emoff)     = diffboolstep  bon   boff    ctx.em   octx.em  "EM" in
      let (ion, ioff)       = diffboolstep  emon  emoff   ctx.i    octx.i   "I"  in
      let (plon, ploff)     = diffboolstep  ion   ioff    ctx.pl   octx.pl  "PL" in
      let (son, soff)       = diffboolstep  plon  ploff   ctx.s    octx.s   "S"  in
      let (tton, ttoff)     = diffboolstep  son   soff    ctx.tt   octx.tt  "TT" in
      let (u1on, u1off)     = diffboolstep  tton  ttoff   ctx.u1   octx.u1  "U"  in
      let (u2on, u2off)     = diffboolstep  u1on  u1off   ctx.u2   octx.u2  "U"  in
      let (u3on, u3off)     = diffboolstep  u2on  u2off   ctx.u3   octx.u3  "U"  in
      let (clron, clroff)   = diffboolstep  u3on  u3off   ctx.clr  octx.clr "r"  in
      let (clgon, clgoff)   = diffboolstep  clron clroff  ctx.clg  octx.clg "g" in
      let (clbon, clboff)   = diffboolstep  clgon clgoff  ctx.clb  octx.clb "b" in
      let (clcon, clcoff)   = diffboolstep  clbon clboff  ctx.clc  octx.clc "c" in
      let (clyon, clyoff)   = diffboolstep  clcon clcoff  ctx.cly  octx.cly "y" in
      let (clmon, clmoff)   = diffboolstep  clyon clyoff  ctx.clm  octx.clm "m" in
      let (clkon, clkoff)   = diffboolstep  clmon clmoff  ctx.clk  octx.clk "k" in
      let (clwon, clwoff)   = diffboolstep  clkon clkoff  ctx.clw  octx.clw "w" in
      let (sz0on, sz0off)   = diffboolstep  clwon clwoff  ctx.z0   octx.z0  "0" in
      let (sz1on, sz1off)   = diffboolstep  clwon clwoff  ctx.z1   octx.z1  "1" in
      let (sz2on, sz2off)   = diffboolstep  clwon clwoff  ctx.z2   octx.z2  "2" in
      let (sz3on, sz3off)   = diffboolstep  clwon clwoff  ctx.z3   octx.z3  "3" in
      let (sz4on, sz4off)   = diffboolstep  clwon clwoff  ctx.z4   octx.z4  "4" in
      let (sz5on, sz5off)   = diffboolstep  clwon clwoff  ctx.z5   octx.z5  "5" in
      let (sz6on, sz6off)   = diffboolstep  clwon clwoff  ctx.z6   octx.z6  "6" in
      let (sz7on, sz7off)   = diffboolstep  clwon clwoff  ctx.z7   octx.z7  "7" in
      let (sz8on, sz8off)   = diffboolstep  clwon clwoff  ctx.z8   octx.z8  "8" in
      let (sz9on, sz9off)   = diffboolstep  clwon clwoff  ctx.z9   octx.z9  "9" in
      let ordering a b =
	let (astr, ai) = a in
	let (bs, bi) = b in
	let orderInt = compare ai bi in
	  if orderInt = 0 then
	    compare astr bs
	  else
	    orderInt in
	(List.sort ordering sz9on, sz9off)
    in
      match table with
	  [] -> !sbuf
	| ((s, ctx)::t) ->
	    let (on, off) = diff ctx octx in
	    let rec print_off list =
	      match list with 
		  [] -> ()
		| _::l -> sbuf:= (!sbuf ^ (Stack.pop stack)); print_off l
	    in
	    let rec push_on list =
	      let open_tag s = "<" ^ s ^ ">" in
	      let close_tag s= "</" ^ s ^ ">" in
		match list with 
		    [] -> ()
		  | (s2,_)::l -> sbuf := (!sbuf ^ (open_tag s2));
		      (Stack.push (close_tag s2) stack);
		      push_on l
	    in
	      print_off off; push_on on; sbuf := (!sbuf ^ s); render2 t ctx stack; !sbuf
  in
    render2 (table @ [("", default)]) default (Stack.create ())
