open Attr

type token = EOF | Text of string | Space of string | BO | EM | I | PL | S 
	     | TT | UL | Z0 | Z1 | Z2 | Z3 | Z4 | Z5 | Z6 | Z7 | Z8 | Z9 
	     | CLR | CLG | CLB | CLC| CLY | CLM | CLK | CLW | E_BO | E_EM 
	     | E_I | E_PL | E_S | E_TT | E_UL | E_Z0 | E_Z1 | E_Z2 | E_Z3 
	     | E_Z4 | E_Z5 | E_Z6 | E_Z7 | E_Z8 | E_Z9 | E_CLR | E_CLG 
	     | E_CLB | E_CLC| E_CLY | E_CLM | E_CLK | E_CLW

let tok2string tok = 
  match tok with
      Text t -> "\"" ^ t ^ "\""
    | Space s -> "\"" ^ s ^ "\""
    | BO      -> "<B>"
    | EM      -> "<EM>"
    | I       -> "<I>"
    | PL      -> "<PL>"
    | S       -> "<S>"
    | TT      -> "<TT>"
    | UL      -> "<UL>"
    | Z0      -> "<0>"
    | Z1      -> "<1>"
    | Z2      -> "<2>"
    | Z3      -> "<3>"
    | Z4      -> "<4>"
    | Z5      -> "<5>"
    | Z6      -> "<6>"
    | Z7      -> "<7>"
    | Z8      -> "<8>"
    | Z9      -> "<9>"
    | CLR     -> "<r>"
    | CLG     -> "<g>"
    | CLB     -> "<b>"
    | CLC     -> "<c>"
    | CLY     -> "<y>"
    | CLM     -> "<m>"
    | CLK     -> "<k>"
    | CLW     -> "<w>"
    | E_BO    -> "</B>"
    | E_EM    -> "</EM>"
    | E_I     -> "</I>"
    | E_PL    -> "</PL>"
    | E_S     -> "</S>"
    | E_TT    -> "</TT>"
    | E_UL    -> "</UL>"
    | E_Z0    -> "</0>"
    | E_Z1    -> "</1>"
    | E_Z2    -> "</2>"
    | E_Z3    -> "</3>"
    | E_Z4    -> "</4>"
    | E_Z5    -> "</5>"
    | E_Z6    -> "</6>"
    | E_Z7    -> "</7>"
    | E_Z8    -> "</8>"
    | E_Z9    -> "</9>"
    | E_CLR   -> "</r>"
    | E_CLG   -> "</g>"
    | E_CLB   -> "</b>"
    | E_CLC   -> "</c>"
    | E_CLY   -> "</y>"
    | E_CLM   -> "</m>"
    | E_CLK   -> "</k>"
    | E_CLW   -> "</w>"
    | EOF     -> "EOF"
	

let tok2ctx tok ctx = 
  match tok with
      BO     -> { ctx with b = (True, 0) }
    | EM     -> 
	let next_em ctx = 
	  let (cs,_) = ctx.s in
	  let (cem,_) = ctx.em in
	    if (cs = True) then ctx
	    else if (cem = True) then { ctx with em = (False, 0) }
	    else { ctx with em = (True, 0) } 
	in (next_em ctx)
	     
      | I      -> { ctx with i = (True, 0) }
      | PL     -> { ctx with b = (False, 0); em = (False, 0); i = (False, 0); 
		       s = (False, 0); tt = (False, 0); u1 = (False, 0); 
		       u2 = (False, 0); u3 = (False, 0) 
		   }
      | S      ->  { ctx with s = (True, 0); em = (False, 0) }
      | TT     ->  { ctx with tt = (True, 0) }
      | UL     ->  
	  let next_ul ctx = 
	    let (cu1,_) = ctx.u1 in
	    let (cu2,_) = ctx.u2 in
	    let (cu3,_) = ctx.u3 in
	      if (cu1 = False) then { ctx with u1 = (True, 0) }
	      else if (cu2 = False) then { ctx with u2 = (True, 0) }
	      else { ctx with u3 = (True, 0) }
	  in
	    (next_ul ctx)	    
      | Z0    -> { ctx with z0 = (True, 0) }
      | Z1    -> { ctx with z1 = (True, 0) }
      | Z2    -> { ctx with z2 = (True, 0) }
      | Z3    -> { ctx with z3 = (True, 0) }
      | Z4    -> { ctx with z4 = (True, 0) }
      | Z5    -> { ctx with z5 = (True, 0) }
      | Z6    -> { ctx with z6 = (True, 0) }
      | Z7    -> { ctx with z7 = (True, 0) }
      | Z8    -> { ctx with z8 = (True, 0) }
      | Z9    -> { ctx with z9 = (True, 0) }
      | CLR    -> { ctx with clr = (True, 0) }
      | CLG    -> { ctx with clg = (True, 0) }
      | CLB    -> { ctx with clb = (True, 0) }
      | CLC    -> { ctx with clc = (True, 0) }
      | CLY    -> { ctx with cly = (True, 0) }
      | CLM    -> { ctx with clm = (True, 0) }
      | CLK    -> { ctx with clk = (True, 0) }
      | CLW    -> { ctx with clw = (True, 0) }
      | _      -> ctx
	  
(* space needs a special context *)
(* it can take almost any value (except for UL and colors if UL is set) *)
let space_ctx ctx ul = 
  ctx
(*
  let c1 = { 
    ctx with b = (Star, 0); em = (Star, 0); i = (Star, 0);  s = (Star, 0); 
	     z0 = (Star, 0); z1 = (Star, 0);  
	     z2 = (Star, 0);  z3 = (Star, 0); z4 = (Star, 0); z5 = (Star, 0); 
	     z6 = (Star, 0);  z7 = (Star, 0);  z8 = (Star, 0);     
	     z9 = (Star, 0) }
  in
    if (ul) then c1 else
      { c1 with 
	  clr = (Star, 0); clg = (Star, 0); clb = (Star, 0); clc = (Star, 0); 
	  cly = (Star, 0); clm = (Star, 0); clk = (Star, 0); clw = (Star, 0)
  }
*)
